@@ -21,7 +21,7 @@ import enum
 import math
 import datetime as dt
 import operator
-from decimal import Context, Decimal
+from decimal import Context, Decimal, localcontext
 from inspect import isclass, isfunction
 from fractions import Fraction
 from functools import reduce
@@ -33,8 +33,8 @@ from hypothesis.internal.cache import LRUReusedCache
 from hypothesis.searchstrategy import SearchStrategy
 from hypothesis.internal.compat import gcd, ceil, floor, hrange, \
     text_type, get_type_hints, getfullargspec, implements_iterator
-from hypothesis.internal.floats import is_negative, float_to_int, \
-    int_to_float, count_between_floats
+from hypothesis.internal.floats import next_up, next_down, is_negative, \
+    float_to_int, int_to_float, count_between_floats
 from hypothesis.internal.charmap import as_general_categories
 from hypothesis.internal.renaming import renamed_arguments
 from hypothesis.utils.conventions import infer, not_set
@@ -334,17 +334,32 @@ def floats(
                     allow_nan
                 ))
 
-    min_value = try_convert(float, min_value, 'min_value')
-    max_value = try_convert(float, max_value, 'max_value')
-
     check_valid_bound(min_value, 'min_value')
     check_valid_bound(max_value, 'max_value')
+
+    min_arg, max_arg = min_value, max_value
+    if min_value is not None:
+        min_value = float(min_value)
+    if max_value is not None:
+        max_value = float(max_value)
+
     check_valid_interval(min_value, max_value, 'min_value', 'max_value')
     if min_value == float(u'-inf'):
         min_value = None
     if max_value == float(u'inf'):
         max_value = None
 
+    if min_value is not None and min_value < min_arg:
+        min_value = next_up(min_value)
+        assert min_value > min_arg
+    if max_value is not None and max_value > max_arg:
+        max_value = next_down(max_value)
+        assert max_value < max_arg
+    if None not in (min_value, max_value) and min_value > max_value:
+        raise InvalidArgument(
+            'There are no floating-point values between min_value=%r and '
+            'max_value=%r' % (min_arg, max_arg))
+
     if allow_infinity is None:
         allow_infinity = bool(min_value is None or max_value is None)
     elif allow_infinity:
@@ -444,7 +459,7 @@ def tuples(*args):
         check_strategy(arg)
 
     from hypothesis.searchstrategy.collections import TupleStrategy
-    return TupleStrategy(args, tuple)
+    return TupleStrategy(args)
 
 
 @defines_strategy
@@ -509,10 +524,19 @@ def lists(
     list, and by shrinking each individual element of the list.
     """
     check_valid_sizes(min_size, average_size, max_size)
-    if elements is None or (max_size is not None and max_size <= 0):
-        if max_size is None or max_size > 0:
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the list will be required '
+            'in a future version of Hypothesis.  To create lists that are '
+            'always empty, use `builds(list)` or `lists(nothing())`.'
+        )
+        if min_size or average_size or max_size:
+            # Checked internally for lists with an elements strategy, but
+            # we're about to skip that and return builds(list) instead...
             raise InvalidArgument(
-                u'Cannot create non-empty lists without an element type'
+                'Cannot create a non-empty collection (min_size=%r, '
+                'average_size=%r, max_size=%r) without elements.'
+                % (min_size, average_size, max_size)
             )
         else:
             return builds(list)
@@ -553,7 +577,7 @@ def lists(
         )
     else:
         return ListStrategy(
-            (elements,), average_length=average_size,
+            elements, average_size=average_size,
             min_size=min_size, max_size=max_size,
         )
 
@@ -570,6 +594,12 @@ def sets(elements=None, min_size=None, average_size=None, max_size=None):
     Examples from this strategy shrink by trying to remove elements from the
     set, and by shrinking each individual element of the set.
     """
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the set will be required '
+            'in a future version of Hypothesis.  To create sets that are '
+            'always empty, use `builds(set)` or `sets(nothing())`.'
+        )
     return lists(
         elements=elements, min_size=min_size, average_size=average_size,
         max_size=max_size, unique=True
@@ -581,6 +611,13 @@ def sets(elements=None, min_size=None, average_size=None, max_size=None):
 def frozensets(elements=None, min_size=None, average_size=None, max_size=None):
     """This is identical to the sets function but instead returns
     frozensets."""
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the frozenset will be '
+            'required in a future version of Hypothesis.  To create '
+            'frozensets that are always empty, use `builds(frozenset)` '
+            'or `frozensets(nothing())`.'
+        )
     return lists(
         elements=elements, min_size=min_size, average_size=average_size,
         max_size=max_size, unique=True
@@ -597,6 +634,13 @@ def iterables(elements=None, min_size=None, average_size=None, max_size=None,
     which cannot be indexed and do not have a fixed length. This ensures
     that you do not accidentally depend on sequence behaviour.
     """
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the iterable will be '
+            'required in a future version of Hypothesis.  To create '
+            'iterables that are always empty, use `iterables(nothing())`.'
+        )
+
     @implements_iterator
     class PrettyIter(object):
         def __init__(self, values):
@@ -1205,6 +1249,25 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
         lambda f: f.limit_denominator(max_denominator))
 
 
+def _as_finite_decimal(value, name, allow_infinity):
+    """Convert decimal bounds to decimals, carefully."""
+    assert name in ('min_value', 'max_value')
+    if value is None:
+        return None
+    if not isinstance(value, Decimal):
+        with localcontext(Context()):  # ensure that default traps are enabled
+            value = try_convert(Decimal, value, name)
+    if value.is_finite():
+        return value
+    if value.is_infinite() and (value < 0 if 'min' in name else value > 0):
+        if allow_infinity or allow_infinity is None:
+            return None
+        raise InvalidArgument('allow_infinity=%r, but %s=%r'
+                              % (allow_infinity, name, value))
+    # This could be infinity, quiet NaN, or signalling NaN
+    raise InvalidArgument(u'Invalid %s=%r' % (name, value))
+
+
 @cacheable
 @defines_strategy_with_reusable_values
 def decimals(min_value=None, max_value=None,
@@ -1233,28 +1296,9 @@ def decimals(min_value=None, max_value=None,
     check_valid_integer(places)
     if places is not None and places < 0:
         raise InvalidArgument('places=%r may not be negative' % places)
-
-    if min_value is not None:
-        min_value = try_convert(Decimal, min_value, 'min_value')
-        if min_value.is_infinite() and min_value < 0:
-            if not (allow_infinity or allow_infinity is None):
-                raise InvalidArgument('allow_infinity=%r, but min_value=%r'
-                                      % (allow_infinity, min_value))
-            min_value = None
-        elif not min_value.is_finite():
-            # This could be positive infinity, quiet NaN, or signalling NaN
-            raise InvalidArgument(u'Invalid min_value=%r' % min_value)
-    if max_value is not None:
-        max_value = try_convert(Decimal, max_value, 'max_value')
-        if max_value.is_infinite() and max_value > 0:
-            if not (allow_infinity or allow_infinity is None):
-                raise InvalidArgument('allow_infinity=%r, but max_value=%r'
-                                      % (allow_infinity, max_value))
-            max_value = None
-        elif not max_value.is_finite():
-            raise InvalidArgument(u'Invalid max_value=%r' % max_value)
+    min_value = _as_finite_decimal(min_value, 'min_value', allow_infinity)
+    max_value = _as_finite_decimal(max_value, 'max_value', allow_infinity)
     check_valid_interval(min_value, max_value, 'min_value', 'max_value')
-
     if allow_infinity and (None not in (min_value, max_value)):
         raise InvalidArgument('Cannot allow infinity between finite bounds')
     # Set up a strategy for finite decimals.  Note that both floating and
